const system = require("../../../system");
const ServiceBase = require("../../sve.base")
const settings = require("../../../../config/settings")
const jwt = require('jsonwebtoken');
class UserService extends ServiceBase {
  constructor() {
    super("auth", ServiceBase.getDaoName(UserService));
    this.companyDao=system.getObject("db.common.companyDao");
    this.roleDao=system.getObject("db.common.roleDao");
  }
 
  //平台注册,默认角色，不具备平台级别的管理或运营功能
  //注册接口
  //封装kongurl
  async pmregister(p,q){
    var self=this;
    let tmppwd=p.password;
    if(!tmppwd){
      tmppwd=settings.defaultpwd;
    }
    p.password=this.getEncryptStr(tmppwd);
    return  this.db.transaction(async function (t) {
      //创建一个默认公司,公司名称xxxx的公司
      let cmp=await self.companyDao.create({name:p.userName+"的公司"},t);
      p.company_id=cmp.id;
      let u=await self.dao.create(p,t)
      //设置默认角色，租户
      //设置默认普通角色,由于有了租户概念,所以注册时，需要知道当前租户和应用的id 才可以设置默认角色 todo
      var role = await self.roleDao.model.findOne({ where: { code: "ta" }, transaction: t });
      await u.setRoles([role], { transaction: t });
       
      //创建统一账号
      let cred=await self.cregister(u.userName,cmp.id,p.password)
      console.log("......................................");
      console.log(cred.consumer.id);
      if(cred){
         u.center_id=cred.consumer.id;
         u.jwtkey=cred.key;
         u.jwtsecret=cred.secret;
         try{
          await u.save({transaction:t});
         }catch(e){
           console.log(e);
          await self.cunregister(p.userName);
          throw new Error("保存用户失败");
         }
        return u;
      }else{
        throw new Error("创建统一账号失败");
      }
    });
  }
   //平台登录
  //登录接口封装kong-url
  //登录路由放行
  //p里面含有appkey,company_id,userName,password
  async pmlogin(p,q,req){
    var self=this;
    //先要按照用户名，在统一账户中查找存在性
    //如果不存在
    let consumer=await this.cget(UserService.consumerUrl(p.userName));
    if(!consumer.data){
        return null;
    }else{
      let password=consumer.data.tags[1].split("_")[1];
      let inpassword=this.getEncryptStr(p.password);
      if(password!=inpassword){
        return null;
      }
    }
    var rtn={}
    return  this.db.transaction(async function (t) {
        let userfind=await self.dao.findOne({userName:p.userName,app_id:settings.pmappid,company_id:settings.pmcompanyid});
        if(userfind){
            let token= await self.cmakejwt(userfind.jwtkey,userfind.jwtsecret,null);
            rtn.token=token;
            rtn.user=userfind;
            return rtn;
        }else{
            return null;
        }
    })
  }
  //平台注册与登录
  //用户验证码登录
  //
  async pmloginByVCode(p,q,req){
    var rtn={}
    //检查传递过来的手机验证码是否与缓存的一致
     let mobile=p.mobile;
     let vcode=p.vcode;
     let cachevcode=await this.cacheManager ["VCodeCache"].getCache(mobile,60)
     if(vcode!=cachevcode.vcode){
       return null;
     }else{
       //检查是否在库里存在
       //appkey--company_id---需要控制器基类里设置到p对象里
       let user=await this.findOne({userName:mobile,app_id:settings.pmappid,company_id:settings.pmcompanyid})
       if(user){
         //生成token
         let token= await this.cmakejwt(user.jwtkey,user.jwtsecret,null);
         rtn.token=token;
         rtn.user=user;
         return rtn;
       }else{
         //先按照用户名查续身份信息，获取key,secret,
         let u=await this.pmregister({userName:mobile,nickName:mobile});
         let token= await this.cmakejwt(u.jwtkey,u.jwtsecret,null);
         rtn.token=token;
         rtn.user=u;
         return rtn;
       }
     }
    //不一致那么就
  }
  //发送手机验证码并缓存
  async sendVCode(p,q,req){
      let mobile=p.mobile;
      let vcodeobj=await this.cacheManager ["VCodeCache"].cache(mobile,null,60);
      return vcodeobj.vcode;
  }
  async reSendVCode(p,q,req){
    let mobile=p.mobile;
    await this.cacheManager ["VCodeCache"].invalidate(mobile);
    let vcodeobj=await this.cacheManager ["VCodeCache"].cache(mobile,null,60);
    return vcodeobj.vcode;
}

  //创建统一账号及jwt身份
  async cregister(uname,cmpid,pass){
    try{
      var rtn2=null;
      let rtn=await system.postJsonTypeReq(UserService.newConsumerUrl(),{username:uname,tags:["cmp_"+cmpid,"pass_"+pass]})
      console.log(rtn)
      if(rtn.statusCode==409){
        throw new Error("已经存在相同的统一账号名称!");
      }else{
          //创建身份
          rtn2=await system.post3wFormTypeReq(UserService.newJwtCredUrl(uname))
      }
      if(rtn.statusCode==201 && rtn2.statusCode==201){
        return rtn2.data;
      }
      return null;
    }catch(e){
      console.log(e);
      return  null;
    }
  }
  async jwtsign(plkey,secretstr,opts){
     let promise=new Promise(function(resv,rej){
         jwt.sign({ iss: plkey },secretstr,opts,function(err,rtn){
           if(err){
             rej(err);
           }else{
            resv(rtn);
           }
         });
     });
     return promise;
  }
  //只要登录 生成新的访问jwttoken
  async cmakejwt(key,secret,opts){
    var token =await  this.jwtsign(key, secret,opts);
    return token;
  }

  //删除统一账号
    async cunregister(uname){
      try{
        let rtn=await system.delReq(UserService.consumerUrl(uname))
        if(rtn.statusCode==204){
          return {};
        }
        return null;
      }catch(e){
        return  null;
      }
    }

  //登录统一账号
  async clogin(uname){
    //检查是否存在重名

  }
  //按用户名查询统一用户
  async findCUser(uname){

  }
}

module.exports = UserService;
// // (async ()=>{
//    let u=new UserService();
// //   //  let x=await u.cregister("jiangong")
// //   //  console.log(x)
// //   // let x=await u.cunregister("jiangong")
// //   //  console.log(x)
// //   // let t=await u.cmakejwt()
// //   // console.log(t)
//   let ux=u.register({userName:"jiangyong3",password:"123"})
//   console.log(ux);
// // })()